@@ -1,16 +1,18 @@
-#   Copyright 2019 AUI, Inc. Washington DC, USA
+#  CASA Next Generation Infrastructure
+#  Copyright (C) 2021 AUI, Inc. Washington DC, USA
 #
-#   Licensed under the Apache License, Version 2.0 (the "License");
-#   you may not use this file except in compliance with the License.
-#   You may obtain a copy of the License at
+#  This program is free software: you can redistribute it and/or modify
+#  it under the terms of the GNU General Public License as published by
+#  the Free Software Foundation, either version 3 of the License, or
+#  (at your option) any later version.
 #
-#       http://www.apache.org/licenses/LICENSE-2.0
+#  This program is distributed in the hope that it will be useful,
+#  but WITHOUT ANY WARRANTY; without even the implied warranty of
+#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#  GNU General Public License for more details.
 #
-#   Unless required by applicable law or agreed to in writing, software
-#   distributed under the License is distributed on an "AS IS" BASIS,
-#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-#   See the License for the specific language governing permissions and
-#   limitations under the License.
+#  You should have received a copy of the GNU General Public License
+#  along with this program.  If not, see <https://www.gnu.org/licenses/>.
 """
 this module will be included in the api
 """
@@ -182,10 +184,13 @@ def read_vis(
         xds_list = []
         for part in partition:
             if part == 'global': continue
-            if os.path.isdir(os.path.join(infile, str(part))):
-                xds_list += [(part.replace("global/", ""), open_zarr(os.path.join(infile, str(part)), chunks=chunks,
-                                                                     consolidated=consolidated,
-                                                                     overwrite_encoded_chunks=overwrite_encoded_chunks))]
+            try:
+                if os.path.isdir(os.path.join(infile, str(part))):
+                    xds_list += [(part.replace("global/", ""), open_zarr(os.path.join(infile, str(part)), chunks=chunks,
+                                                                         consolidated=consolidated,
+                                                                         overwrite_encoded_chunks=overwrite_encoded_chunks))]
+            except:
+                print('Can not open ', part)
     # build the master xds to return
     xds = xdsio.vis_xds_packager(xds_list)
 
